#!/bin/bash


LABEL="Elasticsearch"
SEARCH_GUARD_DIR=/usr/share/elasticsearch/plugins/search-guard-7
: ${ELASTIC_SEARCH_URL:="http://localhost:9200"}
: ${ELASTIC_TRANSPORT_PORT:=9300}

if [ ! -d "${SEARCH_GUARD_DIR}" ]; then
    echo "Unable to find ${SEARCH_GUARD_DIR} (SEARCH_GUARD_DIR) directory, please install secure guard plugin and retry"
    exit 1
fi

log() {
    echo -e "$1"
}

errorExit() {
    echo -e "\033[31mERROR: $1 \033[0m"; echo
    exit 1
}

warn() {
    echo -e "\033[31mWARN: $1 \033[0m"; echo
    exit 1
}

warnMessage() {
    echo -e "\033[31mWARN: $1 \033[0m"; echo
}

waitForElasticsearchToStart() {
    local elasticsearch_health=1
    local max_retry=25
    local counter=0
    until [[ ${elasticsearch_health} == 0 || counter -eq ${max_retry} ]]; do
        ((counter++))
        log "Checking if the elasticsearch is up ... (Attempt: ${counter})"
        curl  -sL -I --output /dev/null "$ELASTIC_SEARCH_URL"
        elasticsearch_health=$?
        sleep 15
    done
    if [[ counter -eq ${max_retry} ]]; then
        warn "Reached maximum attempts (${max_retry}). elasticsearch does not seem to have started"
        return 1
    fi
    return 0
}

setSystemValue(){
    local path=$1
    local value=$2
    local ymlFile=${3}
    local option=${4:-""}
    local displayValue=

    [ -z "${path}"    ] && errorExit "a path is mandatory to modify system yaml"                || true
    [ -z "${value}"   ] && errorExit "a value is mandatory to update a path in system yaml"     || true
    [ -z "${ymlFile}" ] && errorExit "a yaml file is mandatory to update a path in system yaml" || true

    log "setting ${path} with ${displayValue} in ${ymlFile}"
    
    if [[ ! -f "${ymlFile}" || ! -s "${ymlFile}" ]]; then
        value=$value ${YQ_PATH} e -n ${option} "${path} = env(value)" > "${ymlFile}" || errorExit "failed to set create ${path} with ${displayValue} in ${yamlFile}, make sure its parent directory is already create and $USER user has permission to create this file under it,\ncommand used : ${YQ_PATH} e -n ${option} \"${path} = \"${value}\"\" > \"${ymlFile}\""
    else
        value=$value ${YQ_PATH} eval -i ${option} "${path} = env(value)" "${ymlFile}" || errorExit "failed to set create ${path} with ${displayValue} in ${yamlFile}, \ncommand used : ${YQ_PATH} eval -i ${option} \"${path} = \"${value}\"\" \"${ymlFile}\""
    fi
}

updateAnonymousAuthConfig() {
    setSystemValue ".sg_config.dynamic.http.anonymous_auth_enabled" "true" "${SEARCH_GUARD_DIR}/sgconfig/sg_config.yml"
    setSystemValue ".sg_anonymous.cluster_permissions[0]" "cluster:monitor/health" "${SEARCH_GUARD_DIR}/sgconfig/sg_roles.yml"
    setSystemValue ".sg_anonymous.backend_roles[0]" "sg_anonymous_backendrole" "${SEARCH_GUARD_DIR}/sgconfig/sg_roles_mapping.yml"
}

changeElasticSearchUsername () {
    local replaceEsUsername="__REPLACE_ELASTICSEARCH_USERNAME__"
    if [[ ! -z "${ELASTICSEARCH_USERNAME}" ]]; then
        if [[ "${ELASTICSEARCH_USERNAME}" == "${replaceEsUsername}" ]]; then
            return
        fi
        setSystemValue ".${ELASTICSEARCH_USERNAME}.backend_roles[0]" "admin" "${SEARCH_GUARD_DIR}/sgconfig/sg_internal_users.yml"
    fi
}

changeElasticSearchPassword () {
    local replaceEsPassword="__REPLACE_ELASTICSEARCH_PASSWORD__"
    if [[ ! -z "${ELASTICSEARCH_PASSWORD}" ]]; then
        if [[ "${ELASTICSEARCH_PASSWORD}" == "${replaceEsPassword}" ]]; then
            return
        fi
        local hashPassword=$(bash ${SEARCH_GUARD_DIR}/tools/hash.sh -p ${ELASTICSEARCH_PASSWORD})
        setSystemValue ".${ELASTICSEARCH_USERNAME}.hash" "${hashPassword}" "${SEARCH_GUARD_DIR}/sgconfig/sg_internal_users.yml"
    fi
}

bootstrapCustomCertificates () {
    local certsList="localhost.key localhost.pem root-ca.pem sgadmin.key sgadmin.pem"
    local cert=
    local status=
    if [[ -d "/usr/share/elasticsearch/data/certs" ]]; then
        log "Attempting to use custom certificates available under [/usr/share/elasticsearch/data/certs/] "
        for cert in $certsList;
        do
           if [[ -f "/usr/share/elasticsearch/data/certs/${cert}" ]]; then
                status="success"
           else
                status="failure"
                warnMessage "Could not find the custom cert [/usr/share/elasticsearch/data/certs/${cert}]"
                warnMessage "Make sure custom certs [localhost.key localhost.pem root-ca.pem sgadmin.key sgadmin.pem] available under [/usr/share/elasticsearch/data/certs/]"
                break
           fi
        done
    fi

    if [[ "${status}" == "success" ]]; then
        cp -f "/usr/share/elasticsearch/data/certs/localhost.key" "/usr/share/elasticsearch/config/"
        cp -f "/usr/share/elasticsearch/data/certs/localhost.pem" "/usr/share/elasticsearch/config/"
        cp -f "/usr/share/elasticsearch/data/certs/root-ca.pem" "/usr/share/elasticsearch/config/"
        cp -f "/usr/share/elasticsearch/data/certs/root-ca.pem" "/usr/share/elasticsearch/plugins/search-guard-7/tools/"
        cp -f "/usr/share/elasticsearch/data/certs/sgadmin.key" "/usr/share/elasticsearch/plugins/search-guard-7/tools/"
        cp -f "/usr/share/elasticsearch/data/certs/sgadmin.pem" "/usr/share/elasticsearch/plugins/search-guard-7/tools/"
    else
        log "Attempting to use default certificates"
    fi
}

init() {
    bootstrapCustomCertificates
    if [[ "${ELASTICSEARCH_CLUSTERSETUP}" == "NO" ]]; then
        log "Waiting for $LABEL to get ready using the commands: \"curl -sL -I --output /dev/null \"$ELASTIC_SEARCH_URL\"\""
        waitForElasticsearchToStart

        export YQ_PATH=/opt/yq/yq
        export JAVA_HOME=/usr/share/elasticsearch/jdk
        updateAnonymousAuthConfig
        changeElasticSearchUsername
        changeElasticSearchPassword
        log "Initializing secure guard plugin on elasticsearch"
        cd ${SEARCH_GUARD_DIR}/tools && ./sgadmin.sh -p ${ELASTIC_TRANSPORT_PORT} -cacert root-ca.pem -cert sgadmin.pem -key sgadmin.key -nhnv -icl -cd ../sgconfig/ 1>/dev/null
    else
        log "Skipping initialisation of search guard, this will already be intialised in first node"
    fi
}

init
exit $?